
// příklad C99, "variably-modified types"

#include <stdio.h>

#define MAXLINE 1000    // implementační limit

// viz "variably-modified types", možnost kontroly mezí
unsigned get_line(const unsigned lim, char s[lim]) {
    unsigned i = 0;
    int c;              // musí být int
    while ((c = getchar()) != EOF) {
        s[i++] = c;     // uložit znaky včetně '\n'
        if (c == '\n')    break; // konec řádku
        if (i >= lim - 1) break; // nevejde se celý
    }
    s[i] = '\0';        // řetězec ukončí znakem '\0'
    return i;           // vrací délku řetězce
}

// kopie řetězce (bez kontroly mezí)
void copy(char s[], const char s1[]) { // s:=s1
  while( (*s++ = *s1++) != '\0' );
}

int main(void) {
  char save[MAXLINE];   // nejdelší řádek ...
  unsigned max = 0;     // ... a jeho délka
  char line[MAXLINE];   // načtený řádek ...
  unsigned len;         // ... a jeho délka
#ifdef ERROR            // gcc -DERROR
  while( (len=get_line(MAXLINE+1,line)) > 0 )
#else
  while( (len=get_line(MAXLINE,line)) > 0 )
#endif
    if( len > max ) {   // je řádek delší?
      max = len;        // uschovat nový nejdelší
      copy(save, line); // pole předá odkazem
    }
  if( max > 0 )         // neprázdný vstup?
    printf("%s", save);
}

