
// compile:  gcc -std=c99 -Wall -O2 mandelbrot.c -o mandelbrot -lm
// run:      mandelbrot 1024 > out.pbm
// output format: 1bit PBM

// source: http://shootout.alioth.debian.org/

#include <stdio.h>
#include <stdlib.h>
#include <complex.h>            // C99

const double limit = 2.0;

int main(int argc, char *argv[])
{
    int w=256, h=256;
    int bit_num = 0;
    unsigned char byte_acc = 0;

    if(argc>1)
        h = w = atoi(argv[1]);

    printf("P4\n%d %d\n", w, h);

    for (int y = 0; y < h; y++) {
        for (int x = 0; x < w; x++) {
            const int iter = 50; //256;
            complex double Z = 0;
            complex double C = x * 2.0/w - 1.5 + I * (y * 2.0/h - 1);
            int i;

            for (i = 0; i < iter; i++) {
                Z = Z*Z + C;
                if (cabs(Z) > limit)
                    break;
            }

            unsigned char pixel = (i==iter);
            byte_acc = (byte_acc << 1) | pixel;

            bit_num++;

            if (bit_num == 8) {
                putc(byte_acc, stdout);
                byte_acc = 0;
                bit_num = 0;
            } else if (x == w - 1) {
                byte_acc = byte_acc << (8 - w % 8);
                putc(byte_acc, stdout);
                byte_acc = 0;
                bit_num = 0;
            }
        } // for
    } // for

    return 0;
}
